==> release.12.2.28.1.tgz: cgi-cpn/prontus_tags_import_real.pl <==
#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------

# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# PROPOSITO.
# -----------
# Importar tags a Prontus, desde un arch. xml que debe encontrarse en
# $prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/procs/tags_import.xml
# El formato del archivo es el sgte:
# <?xml version='1.0' encoding='utf-8'?>
# <tagsdata>
# <tag id='58'>
#   <nom>eaouaopñp</nom>
#   <count>0</count>
#   <mostrar>SI</mostrar>
# </tag>
# <tag id='59'>
#   <nom>aoeuaoa</nom>
#   <count>0</count>
#   <mostrar>SI</mostrar>
# </tag>
# ...
# </tagsdata>
#
# (más eventuales campos de multivistas).
# ---------------------------------------------------------------
# LLAMADAS A ARCHIVOS EXTERNOS.
# ------------------------------
# No registra.
# ---------------------------------------------------------------
# INVOCACIONES ACEPTADAS.
# ------------------------
# Via system desde prontus_tags_import.cgi
# Esta CGI espera que
# ---------------------------------------------------------------
# PLANTILLAS HTML UTILIZADAS.
# ----------------------------
# No utiliza.
# ---------------------------------------------------------------
# ---------------------------------------------------------------
# TABLAS UTILIZADAS.
# -------------------
# ART (BD prontus)
# ---------------------------------------------------------------

# -------------------------------BEGIN SCRIPT---------------
# ---------------------------------------------------------------
# DIRECTIVAS DE COMPILACION.
# ------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
}

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use DBI;
use File::Copy;
use JSON;
use LockFile::Simple;
use XML::Parser;
use glib_dbi_03;
use glib_fildir_03;
use glib_hrfec_03;
use glib_str_03;
use lib_bd;
use lib_lock;
use lib_logproc;
use lib_parser;
use lib_prontus;
use lib_tags;
use lib_utils;
##/libs##

# ---------------------------------------------------------------
# MAIN.
# -------------
my ($BD, $LOCK_FILE);

my ($TOT_REGS, $OK_REGS) = '0';
my (%NOMBRES);
main:{
    lib_utils::setUtf8();
    my $PRONTUS_ID;

    if (scalar @ARGV < 2) {
        print "Parametros insuficientes Terminando\n";
        print "Uso: $0 [path_server] [nombre_prontus]\nPropósito: importar un archivo xml con los Tags.\n";
        exit;
    }

    if ($ARGV[0] && $ARGV[1]) {
        $prontus_varglb::DIR_SERVER = $ARGV[0];
        $PRONTUS_ID = $ARGV[1];
        # print STDERR "DIR_SERVER: [$prontus_varglb::DIR_SERVER]\n";
        # print STDERR "PRONTUS_ID: [$PRONTUS_ID]\n";
    }

    if (!-d $prontus_varglb::DIR_SERVER) {
        print STDERR "ERROR: DIR_SERVER no válido[$prontus_varglb::DIR_SERVER]\n";
        exit;
    }

    # valida el identificador de prontus
    if ($PRONTUS_ID eq '' || !lib_utils::valida_prontus($PRONTUS_ID)) {
        print "El identificador de Prontus es inválido\n";
        exit;
    }

    # Carga variables de configuracion.
    my $msg_load = lib_utils::load_config_by_id($PRONTUS_ID);
    if ($msg_load ne '') {
        print $msg_load ,"\n";
        exit;
    }

    $lib_logproc::LOG_FILE = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/procs/prontus_tags_import_log.html";
    $lib_logproc::MODO_WEB = 0;

    # Bloqueo
    $LOCK_FILE = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/procs/semaforo_tags_import";

    # Detecta semaforo.
    my $lock_obj = &lib_lock::lock_file($LOCK_FILE);
    if (!ref $lock_obj) { # si ya tiene un bloqueo anterior, aborta.
        &finishLoading("Proceso en ejecución.\nPor favor espere hasta que la importación anterior termine.");
        lib_logproc::handle_error("Proceso en ejecución.\nPor favor espere hasta que la importación anterior termine.");
    }

    # Mysql
    my ($msg_err_bd);
    ($BD, $msg_err_bd) = lib_bd::conectar_prontus_bd();
    if (!ref($BD)) {
        &finishLoading("ERROR: $msg_err_bd");
        lib_logproc::handle_error("ERROR: $msg_err_bd");
    }


    lib_logproc::flush_log();
    lib_logproc::writeRule();
    lib_logproc::add_to_log_count("INICIANDO PROCESO DE IMPORTACION");

    my ($data_file) = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/procs/tags_import.xml";
    my $archivo = glib_fildir_03::read_file($data_file);
    my ($err) = lib_prontus::valid_xml_import($archivo);
    if ($err) {
        &finishLoading("El archivo ingresado no es un XML bien formado, no es posible cargarlo");
        lib_logproc::handle_error("El archivo ingresado no es un XML bien formado, no es posible cargarlo");
    }

    lib_logproc::add_to_log_count("Borrando datos actuales de la base de datos.\n");
    my ($sql) = "delete from TAGS";
    $BD->do($sql) || (&finishLoading("No fue posible borrar los TAGS actuales") && lib_logproc::handle_error("No fue posible borrar Tags") );

    &tags_import($archivo);

    lib_logproc::add_to_log_count("PROCESO DE IMPORTACION FINALIZADO");
    lib_logproc::writeRule();

    $TOT_REGS = '0' if ($TOT_REGS eq '');
    $OK_REGS = '0' if ($OK_REGS eq '');

    lib_logproc::add_to_log("Nro. de registros procesados: $TOT_REGS\nRegistros ingresados OK: $OK_REGS");
    lib_logproc::add_to_log_finish("Operaci&oacute;n finalizada.");

    lib_tags::generar_json($BD);

    # Generar mapa de tags.
    lib_tags::make_mapa_tags('', $BD);

    foreach my $mv (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTIVISTAS'}}) {
        lib_tags::make_mapa_tags($mv, $BD);
    }

    $BD->disconnect;

    &finishLoading('');

    # Elimina el bloqueo y termina ejecucion.
    &lib_lock::unlock_file($lock_obj, $LOCK_FILE);
}

# ---------------------------------------------------------------
# SUB-RUTINAS.
# ---------------------------------------------------------------
sub get_id_mayor {
    my ($buffer_xml) = $_[0];
    my $id_mayor = 0;

    # Obtiene el mayor id q venga de seccion o de tema o de subtema
    while ($buffer_xml =~ /<tag +id *= *["']([0-9]+)["']>.+?<\/tag>/isg) {
        my $id = $1;
        next if ($id !~ /^[0-9]+$/);
        $id_mayor = $id if ($id > $id_mayor);
    }

    return $id_mayor;
}

# ---------------------------------------------------------------
sub tags_import {

    my ($buffer_xml) = $_[0];
    my ($sql, $ret, $linea);

    lib_logproc::add_to_log_count("Procesando archivo de datos.");

    $buffer_xml =~ s/(\r\n)/\n/g;
    $buffer_xml =~ s/\r//g;

    my (@data, $i, %tags, @filas);
    my ($id, $nom, $count, $mostrar);

    # obtiene los id mayores de cada tags level para poder asignar id a los reg. nuevos
    my $ID_MAYOR = &get_id_mayor($buffer_xml);
    $TOT_REGS = 0;

    while ($buffer_xml =~ /<tag +id *= *["']([0-9]*)["']>(.+?)<\/tag>/isg) {
        $id = $1;
        my $tag = $2;

        # si no viene id asigno uno nuevo
        if (!$id) {
            $ID_MAYOR++;
            $id = $ID_MAYOR;
        }

        next if (!&procesar_xml($id, $tag));
    }
}

# ---------------------------------------------------------------
sub procesar_xml {

    my ($id_level, $xml_segment) = @_;

    $TOT_REGS++;

    # Si viene id deben ser solo digitos.
    if ($id_level !~ /^[0-9]*$/) {
        $xml_segment=~s/>/&gt;/g;              # >
        $xml_segment=~s/</&lt;/g;              # <
        &saltar_reg("Id no válido en registro [$TOT_REGS] id=$id_level" . $xml_segment);
        &saltar_reg($xml_segment);
        return 0;
    }

    my ($nom, %nom_vistas, $count, $mostrar, $nom4vistas, $port);

    # nom
    if ($xml_segment =~ /<nom>(.*?)<\/nom>/is) {
        $nom = lib_prontus::despulga_item_tag($1);
        $nom = lib_utils::unescape_xml($nom);

    }
    if ($nom !~ /\w+/) {
        $xml_segment=~s/>/&gt;/g;              # >
        $xml_segment=~s/</&lt;/g;              # <
        &saltar_reg("Nombre no válido en registro [$TOT_REGS] id=$id_level" . $xml_segment);
        return 0;
    }
    if (exists $NOMBRES{"$nom"}) {
        $xml_segment=~s/>/&gt;/g;              # >
        $xml_segment=~s/</&lt;/g;              # <
        &saltar_reg("Nombre repetido en registro [$TOT_REGS] id=$id_level" . $xml_segment);
        return 0;
    } else {
        $NOMBRES{"$nom"} = 1;
    }

    # count
    if ($xml_segment =~ /<count>(.*?)<\/count>/is) {
        $count = lib_prontus::despulga_item_tag($1);
    }
    $count =~ s/[^\d]//g;
    if (!$count || $count !~ /^[\d]+$/) {
        $count = 0;
    }

    # mostrar
    if ($xml_segment =~ /<mostrar>(.*?)<\/mostrar>/is) {
        $mostrar = lib_prontus::despulga_item_tag($1);
    }

    if ((!$mostrar) || ($mostrar =~ /no/i)) {
        $mostrar = '';
    } else {
        $mostrar = '1';
    }

    # nombres de taglevel en otras vistas
    foreach my $mv (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTIVISTAS'}}) {
        my $nom_envista;

        if ($xml_segment =~ /<nom-$mv>(.*?)<\/nom-$mv>/is) {
            $nom_envista = $1;
            $nom_envista = lib_prontus::despulga_item_tag($nom_envista);
            $nom_envista = lib_utils::unescape_xml($nom_envista);
        }

        $nom_envista = $nom if (!$nom_envista);
        $nom4vistas .= "$mv\t$nom_envista\n";
    }

    $nom4vistas =~ s/\n$//; # elimina \n sobrante

    # url custom
    if ($xml_segment =~ /<port>(.*?)<\/port>/is) {
        $port = lib_prontus::despulga_item_tag($1);
    }

    if ((!$port) || ($port =~ /no/i)) {
        $port = '';
    }

    # inserta
    my $ret;
    $ret = &insert_tag($id_level, $nom, $count, $mostrar, $nom4vistas, $port);
    if ($ret) {
        $OK_REGS++;  # Total de reg. insertados normalmente
    } else {
        $xml_segment=~s/>/&gt;/g;              # >
        $xml_segment=~s/</&lt;/g;              # <
        lib_logproc::add_to_log_count("INSERT error en registro [$TOT_REGS] id=$id_level ($DBI::errstr) $xml_segment");
        return 0;
    }

    return 1;
}

# ---------------------------------------------------------------
sub insert_tag {
    my ($id, $nom, $count, $mostrar, $nom4vistas, $port) = @_;
    my ($sql, $ret);

    # Se hace la inserción
    $sql = "insert into TAGS (TAGS_ID, TAGS_TAG, TAGS_COUNT, TAGS_MOSTRAR, TAGS_NOM4VISTAS, TAGS_PORT) values(?, ?, ?, ?, ?, ?)";
    my @params = ($id, $nom, $count, $mostrar, $nom4vistas, $port);
    $ret = glib_dbi_03::ejecutar_sql_param($BD, $sql, \@params);

    return $ret;
}

# ---------------------------------------------------------------
sub saltar_reg {
    my ($registro) = $_[0];
    lib_logproc::add_to_log_count("Registro con errores, se omite.\n$registro");
}

# ---------------------------------------------------------------
sub parse_comillas {
    my ($str) = $_[0];
    if ($str eq '""') {
        return '';
    } else {
        $str =~ s/""/'/sg;
        $str =~ s/^"//s;
        $str =~ s/"$//s;
        return $str;
    }
}

# ---------------------------------------------------------------
sub finishLoading {
    my $msg = $_[0];
    my $result_file = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/procs/result_tags_import.json";
    $msg = '{"status":1, "msg":"'.$msg.'"}';
    glib_fildir_03::write_file("$prontus_varglb::DIR_SERVER$result_file", $msg);
}

==> release.12.3.1.2.tgz: cgi-cpn/prontus_tags_import_real.pl <==
#!/usr/bin/perl

# ---------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# ---------------------------------------------------------------

# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# PROPOSITO.
# -----------
# Importar tags a Prontus, desde un arch. xml que debe encontrarse en
# $prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/procs/tags_import.xml
# El formato del archivo es el sgte:
# <?xml version='1.0' encoding='utf-8'?>
# <tagsdata>
# <tag id='58'>
#   <nom>eaouaopñp</nom>
#   <count>0</count>
#   <mostrar>SI</mostrar>
# </tag>
# <tag id='59'>
#   <nom>aoeuaoa</nom>
#   <count>0</count>
#   <mostrar>SI</mostrar>
# </tag>
# ...
# </tagsdata>
#
# (más eventuales campos de multivistas).
# ---------------------------------------------------------------
# LLAMADAS A ARCHIVOS EXTERNOS.
# ------------------------------
# No registra.
# ---------------------------------------------------------------
# INVOCACIONES ACEPTADAS.
# ------------------------
# Via system desde prontus_tags_import.cgi
# Esta CGI espera que
# ---------------------------------------------------------------
# PLANTILLAS HTML UTILIZADAS.
# ----------------------------
# No utiliza.
# ---------------------------------------------------------------
# ---------------------------------------------------------------
# TABLAS UTILIZADAS.
# -------------------
# ART (BD prontus)
# ---------------------------------------------------------------

# -------------------------------BEGIN SCRIPT---------------
# ---------------------------------------------------------------
# DIRECTIVAS DE COMPILACION.
# ------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
}

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use DBI;
use File::Copy;
use JSON;
use LockFile::Simple;
use XML::Parser;
use glib_dbi_03;
use glib_fildir_03;
use glib_hrfec_03;
use glib_str_03;
use lib_bd;
use lib_encodes;
use lib_lang;
use lib_lock;
use lib_logproc;
use lib_parser;
use lib_prontus;
use lib_setbd;
use lib_tags;
use lib_utils;
##/libs##

# ---------------------------------------------------------------
# MAIN.
# -------------
my ($BD, $LOCK_FILE);
my ($TOT_REGS, $OK_REGS) = (0, 0);
my (%NOMBRES);

main:{
    lib_utils::setUp();
    my $PRONTUS_ID;
    if (scalar @ARGV < 2) {
        print lib_lang::get_message('global_insufficient_arguments');
        print lib_lang::get_interpolated_message("tags_import_real_use", $0);
        exit;
    }

    if ($ARGV[0] && $ARGV[1]) {
        $prontus_varglb::DIR_SERVER = $ARGV[0];
        $PRONTUS_ID = $ARGV[1];
        # print STDERR "DIR_SERVER: [$prontus_varglb::DIR_SERVER]\n";
        # print STDERR "PRONTUS_ID: [$PRONTUS_ID]\n";
    }

    if (!-d $prontus_varglb::DIR_SERVER) {
        print STDERR lib_lang::get_message('global_error_invalid_dir')."\n";
        exit;
    }

    # valida el identificador de prontus
    if ($PRONTUS_ID eq '' || !lib_utils::valida_prontus($PRONTUS_ID)) {
        print lib_lang::get_message('global_invalid_prontus_id') . "\n";
        exit;
    }

    # Carga variables de configuracion.
    my $msg_load = lib_utils::load_config_by_id($PRONTUS_ID);
    if ($msg_load ne '') {
        print $msg_load ,"\n";
        exit;
    }

    $lib_logproc::LOG_FILE = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/procs/prontus_tags_import_log.html";
    $lib_logproc::JS_FILE  = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/procs/result_tags_import.json";

    # Bloqueo
    $LOCK_FILE = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/procs/semaforo_tags_import";

    # Detecta semaforo.
    my $lock_obj = &lib_lock::lock_file($LOCK_FILE);
    if (!ref $lock_obj) { # si ya tiene un bloqueo anterior, aborta.
        lib_logproc::finishLoading(lib_lang::get_message('global_previous_import'));
        lib_logproc::handle_error(lib_lang::get_message('global_previous_import'));
    }

    # Mysql
    my ($msg_err_bd);
    ($BD, $msg_err_bd) = lib_bd::conectar_prontus_bd();
    if (!ref($BD)) {
        lib_logproc::finishLoading($msg_err_bd);
        lib_logproc::handle_error($msg_err_bd);
    }


    lib_logproc::writeRule();
    lib_logproc::add_to_log_count(lib_lang::get_message('global_import_starting'));

    my ($data_file) = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN'}/procs/tags_import.xml";
    my $archivo = glib_fildir_03::read_file($data_file);
    my ($err) = lib_prontus::valid_xml_import($archivo);
    if ($err) {
        lib_logproc::finishLoading(lib_lang::get_message('global_import_error_invalid_xml'));
        lib_logproc::handle_error(lib_lang::get_message('global_import_error_invalid_xml'));
    }

    # se dropea la tabla de secc
    my $table_name = lib_utils::get_table('tags');
    if (lib_setbd::existe_tabla($BD, $table_name)) {
        my $sql = "drop table $table_name";
        glib_dbi_03::ejecutar_sql($BD, $sql);
        if ($BD->errstr) {
            print STDERR lib_lang::get_message('global_error_table_delete') ." $table_name [$sql][$!][".$BD->errstr."]\n";
            my $msg_delete = lib_lang::get_message('tags_import_real_error_deleting');
            lib_logproc::finishLoading($msg_delete);
            lib_logproc::handle_error($msg_delete);
        }
    }

    # Re-crear estructura de la tabla
    my ($msg_ret, $hay_err) = lib_setbd::crear_tabla_tags($BD);
    lib_logproc::add_to_log(lib_lang::get_message($msg_ret));
    if ($hay_err) {
        lib_logproc::finishLoading(lib_lang::get_interpolated_message('regenerabd_real_fail_creating_structure', 'tags'));
        lib_logproc::handle_error(lib_lang::get_interpolated_message('regenerabd_real_fail_creating_structure', 'tags'));
    }

    &tags_import($archivo);

    lib_logproc::add_to_log_count(lib_lang::get_message('global_import_ending'));
    lib_logproc::writeRule();

    $TOT_REGS = '0' if ($TOT_REGS eq '');
    $OK_REGS = '0' if ($OK_REGS eq '');

    lib_logproc::add_to_log(lib_lang::get_interpolated_message('global_processed_status_correct', $TOT_REGS, $OK_REGS));
    lib_logproc::add_to_log_finish(lib_lang::get_message('global_finalized'));

    lib_tags::generar_json($BD);

    # Generar mapa de tags.
    lib_tags::make_mapa_tags('', $BD);

    foreach my $mv (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTIVISTA'}}) {
        lib_tags::make_mapa_tags($mv, $BD);
    }

    $BD->disconnect;

    lib_logproc::finishLoading('');

    # Elimina el bloqueo y termina ejecucion.
    &lib_lock::unlock_file($lock_obj, $LOCK_FILE);
}

# ---------------------------------------------------------------
# SUB-RUTINAS.
# ---------------------------------------------------------------
sub get_id_mayor {
    my $buffer_xml = $_[0];
    my $id_mayor = 0;

    # Obtiene el mayor id q venga
    while ($buffer_xml =~ /<tag +id *= *["']([0-9]+)["']>.+?<\/tag>/isg) {
        my $id = $1;
        next if ($id !~ /^[0-9]+$/);
        $id_mayor = $id if ($id > $id_mayor);
    }

    return $id_mayor;
}

# ---------------------------------------------------------------
sub tags_import {

    my $buffer_xml = $_[0];
    my ($sql, $ret, $linea);

    lib_logproc::add_to_log_count(lib_lang::get_message('global_import_processing_file') );

    $buffer_xml =~ s/(\r\n)/\n/g;
    $buffer_xml =~ s/\r//g;

    my (@data, $i, %tags, @filas);
    my ($id, $nom);

    # obtiene los id mayores de cada tags level para poder asignar id a los reg. nuevos
    my $ID_MAYOR = &get_id_mayor($buffer_xml);
    $TOT_REGS = 0;

    while ($buffer_xml =~ /<tag +id *= *["']([0-9]*)["']>(.+?)<\/tag>/isg) {
        $id = $1;
        my $tag = $2;

        # si no viene id asigno uno nuevo
        if (!$id) {
            $ID_MAYOR++;
            $id = $ID_MAYOR;
        }

        next if (!&procesar_xml($id, $tag));
    }
}

# ---------------------------------------------------------------
sub procesar_xml {

    my ($id_level, $xml_segment) = @_;

    $TOT_REGS++;

    # Si viene id deben ser solo digitos.
    if ($id_level !~ /^[0-9]*$/) {
        $xml_segment=~s/>/&gt;/g;              # >
        $xml_segment=~s/</&lt;/g;              # <
        &saltar_reg(lib_lang::get_message('global_import_invalid_id') . " [$TOT_REGS] id=$id_level" . $xml_segment);
        &saltar_reg($xml_segment);
        return 0;
    }

    my ($nom, %nom_vistas, $mostrar, $nom4vistas, $port);

    # nom
    if ($xml_segment =~ /<nom>(.*?)<\/nom>/is) {
        $nom = lib_prontus::despulga_item_tag($1);
        $nom = lib_encodes::unescape_xml($nom);

    }
    if ($nom !~ /\w+/) {
        $xml_segment=~s/>/&gt;/g;              # >
        $xml_segment=~s/</&lt;/g;              # <
        &saltar_reg(lib_lang::get_message('global_import_invalid_name') . "[$TOT_REGS] id=$id_level" . $xml_segment);
        return 0;
    }
    if (exists $NOMBRES{"$nom"}) {
        $xml_segment=~s/>/&gt;/g;              # >
        $xml_segment=~s/</&lt;/g;              # <
        &saltar_reg(lib_lang::get_message('global_import_duplicated_name') . " [$TOT_REGS] id=$id_level" . $xml_segment);
        return 0;
    } else {
        $NOMBRES{"$nom"} = 1;
    }

    # mostrar
    if ($xml_segment =~ /<mostrar>SI<\/mostrar>/i) {
        $mostrar = 1;
    } else {
        $mostrar = 0;
    }

    # nombres de taglevel en otras vistas
    foreach my $mv (keys %{$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'MULTIVISTA'}}) {
        my $nom_envista;

        if ($xml_segment =~ /<nom-$mv>(.*?)<\/nom-$mv>/is) {
            $nom_envista = $1;
            $nom_envista = lib_prontus::despulga_item_tag($nom_envista);
            $nom_envista = lib_encodes::unescape_xml($nom_envista);
        }

        $nom_envista = $nom if (!$nom_envista);
        $nom4vistas .= "$mv\t$nom_envista\n";
    }

    $nom4vistas =~ s/\n$//; # elimina \n sobrante

    # url custom
    if ($xml_segment =~ /<port>(.*?)<\/port>/is) {
        $port = lib_prontus::despulga_item_tag($1);
    }

    if ((!$port) || ($port =~ /no/i)) {
        $port = '';
    }

    # inserta
    my $ret = lib_tags::insert_tag($BD, $id_level, $nom, $mostrar, $nom4vistas, $port);
    if ($ret) {
        $OK_REGS++;  # Total de reg. insertados normalmente
    } else {
        $xml_segment=~s/>/&gt;/g;              # >
        $xml_segment=~s/</&lt;/g;              # <
        lib_logproc::add_to_log_count(lib_lang::get_message('global_import_error_insert') . " [$TOT_REGS] id=$id_level ($DBI::errstr) $xml_segment");
        return 0;
    }

    return 1;
}

# ---------------------------------------------------------------
sub saltar_reg {
    my $registro = $_[0];
    lib_logproc::add_to_log_count(lib_lang::get_interpolated_message('global_import_registry_error', $registro));
}
